;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  
;; A Monty Mole as they appeared in Paper Mario
;;   Sit in the ground and throw rocks and bob-ombs every now and then,
;;   similar to Rocky Wrenches but more SMW styled. The bob-ombs don't act like
;;   the SMW ones, they just explode on contact with anything and flash red.
;;
;;  by leod
;;
;;
;;  The enclosed graphics partially replace the Sumo Bros., as that
;;  was the biggest sprite in the Monty Mole set and allowed the explosions
;;  to work as they should.
;;
;;
;;  If you want to use the Bob-Omb or Rock projectile in other sprites,
;;  just spawn this sprite with !Type,x ($151C,x) set to 1 for the Rock
;;  or 2 for the Bob-Omb.
;;  Also set !ProjDex,x to $01 to enable its gravity and floor collision,
;;  keep it at $00 to manipulate it from inside another sprite.
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


!MoleTiles = $C1
;top left tile of the mole
;the hand tiles have to be in the same order right next to it

!BobTile = $A7
;tile to use for the bob-omb

!RockTile = $A9
;tile to use for the rock





;anti magic numbers initiative

!ArmRest = $07
!ArmRaise = $06

!Hidden = $10



;don't change these defines. or do, but it won't help much, they're just random sprite tables

!Type = $151C         ;what "type" the sprite is, so not more than one sprite has to be inserted
                      ;00 = Mole
                      ;01 = Rock
                      ;02 = Bob-Omb

!Dir = $187B          ;direction the sprite is facing
                      ;01 = right
                      ;00 = left

!DirtDir = $1528      ;which side the dirt is facing

!Dead = $C2           ;whether or not moley woley is dead

!ProjDex = $1602      ;the index of the projectile sprite

!ThrowTimer = $1540   ;auto decreases!! timer used for throwing animation



!Buried = $1510       ;how far in the ground moley woley is

!ArmY = $1534         ;how far down the arms are from the body

!ArmDex = $157C       ;which pair of arms to show
                      ;00 = idle hands
                      ;01 = holding projectile
                      ;02 = throw animation left
                      ;03 = throw animation right

!ArmGoal = $1570      ;position the arms want to be at right now
                      ;this is there because arms move more than monty himself



;state machine
!Timer = $1564        ;timer used for many states

!Cargo = $160E        ;miscellaneous stuff to carry from state to state
                      ;00  state to switch to after

!State = $1594        ;mole's state
                      ;00 = waiting
                      ;01 = underground
                      ;02 = emerging
                      ;03 = lowering hands
                      ;04 = lifting projectile
                      ;05 = throwing
                      ;06 = submerging
                      ;07 = dead (falls down if not #$10 or higher)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
LDA #!Hidden
STA !ArmY,x
STA !ArmGoal,x
STA !Buried,x

INC !State,x
STZ !Cargo,x

JSR SubHorzPos  ;store direction
TYA
EOR #$01
STA !Dir,x
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
print "MAIN ",pc
PHB                     ; \
PHK                     ;  | The same old
PLB                     ; /
JSR MainSprtRt          ;  Jump to sprite code
PLB                     ; Yep
RTL                     ; Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  Return1:
RTS

  MainSprtRt:
JSR SubGFX          ;render graphics

LDA $9D             ;if sprites are locked, return
BNE Return1

JSR SubOffScreen_X0	;handle off screen situations

LDA !Type,x
BEQ MoleBehaviour       ;as a mole, be a mole
JMP ProjectileBehaviour ;if it's a projectile, act like one

MoleBehaviour:
;;close arms in on destination
LDA $14
AND #$01          ;every second frame
BNE .ArmsThere

LDA !ArmGoal,x
SEC
SBC !ArmY,x        ;compare current arm offset to the goal
BEQ .ArmsThere     ;if there, cool
BPL .ArmINC        ;if below, increase it

DEC !ArmY,x        ;if above, decrease it
BRA .ArmsThere

  .ArmINC
INC !ArmY,x
  .ArmsThere

;;interaction with mario
LDA !Dead,x
BEQ +
JMP .NoKicked
+

LDA #$0E        ;height (from 1 pixel from top border to very bottom)
SEC
SBC !Buried,x   ;substract how far we are into the floor
BPL .StoreHeight

JMP .NoKicked   ;if that's negative, completely skip interaction with everything

  .StoreHeight


JSL $03B664           ;fill in mario clipping into slot B

;fill mole into slot A
LDA $E4,x       ;x low
CLC
ADC #$02
STA $04
LDA $14E0,x     ;x hi
ADC #$00
STA $0A


LDA !Buried,x
INC
CLC
ADC $D8,x
STA $05
LDA $14D4,x     ;y hi
ADC #$00
STA $0B

LDA #$0C        ;width  (2 pixels from both edges)
STA $06
LDA #$0E        ;height (from 1 pixel from top border to very bottom)
SEC
SBC !Buried,x
STA $07


JSL $03B72B           ;check if the filled in objects intersect
BCS +
JMP .NoContact
+


LDA $1490
ORA $13ED
BEQ .NoStar

JSR Score3

;if mario has a star (or mole gets spinjumped), show puff of smoke and hide mole instantly
  .PuffDeath
JSR Smoke             ;display smoke

LDX $15E9
JSR Death

LDA #!Hidden
STA !ArmY,x
STA !ArmGoal,x        ;hide hands
STA !Buried,x         ;hide mole body

LDA #$07              ;since this death is instantaneous, we might as well go be dead
STA !State,x

BRA .NoContact        ;return after killing the sprite either way


  .NoStar
LDA $7D               ;check if hes falling
BPL +
BRA .HurtMario
+

;move up mario's hitbox by his speed to check for top contact
LDA $01
SEC
SBC $7D
STA $01
LDA $09
SBC #$00
STA $09
JSL $03B72B           ;check if the filled in objects intersect
BCC +
BRA .HurtMario        ;if yes, mario isnt on top, so hurt him
+

JSL $01AA33           ;set mario speed

LDA $140D
ORA $187A             ;if mario is on yoshi or spinjumping, don't do the standard jump
BEQ .NormalJump

JSR Score             ;display score image and play sound
LDA #$1F              ;set spin jump animation timer
STA $1540,x
JSL $07FC3B           ;show star animation
LDA #$08              ;play sound effect
STA $1DF9             ;(overwrites score sound)

BRA .PuffDeath


;jump up and hurt enemy
  .NormalJump
JSR Death


JSL $01AB99           ;display contact graphic
JSR Score             ;display score image and play sound

BRA .NoContact

  .HurtMario
LDA $1497           ;if mario is invincible
BNE .NoContact      ;then skip

LDA $187A
BEQ .DoHurt         ;skip this if mario isnt riding yoshi

JSR LoseYoshi

BRA .NoContact

  .DoHurt
JSL $00F5B7         ;hurt mario

  .NoContact

;; KICKED SPRITE INTERACTION
LDY #$0B          ;initiate loop
        
  .KickLoop

LDA $14C8,y
CMP #$0A          ;if state is not kicked or carried, skip
BCC .next

LDA $009E,y       ;dont interact with springs etc (blacklist because of custom sprites)
CMP #$2F          ;spring
BEQ .next
CMP #$3E          ;switch
BEQ .next
CMP #$2D          ;baby yoshi
BEQ .next
CMP #$80          ;key
BEQ .next

TYX
JSL $03B6E5       ;get that sprite's clip values
LDX $15E9              
JSL $03B72B       ;and check if they intersect (mole is in slot A from mario interaction)
BCC .next         ;if they dont, skip

LDA $14C8,y       ;seems redundant, but the former check was so we dont constantly do the clipping routine
CMP #$0A
BEQ .NoKillSprite

LDA #$02          ;kill sprite
STA $14C8,y

LDA #$DE          ;throw it up a tad
STA $00AA,y

  .NoKillSprite
;we dont kill the kicked sprite, but we kill mole
JSR Score2        ;shell kill score
JSR Smoke         ;display smoke

JSR Death

LDA #!Hidden
STA !ArmY,x
STA !ArmGoal,x    ;hide hands
STA !Buried,x     ;hide mole body

LDA #$07          ;since this death is instantaneous, we might as well go be dead
STA !State,x

  .next
DEY
BPL .KickLoop
  .NoKicked




;; STATE MACHINE
LDA !State,x
ASL
PHX
TAX
JMP.w (.MoleStates,x) 

.MoleStates
dw Wait,Emerge,Lower,Spawn,Lift,Throw,Submerge,Dead
;00 = waiting
;01 = emerging
;02 = lowering hands (and body a bit too when hands are hidden to show effort)
;03 = spawn projectile (one frame)
;04 = lifting projectile (raising up again afterwards)
;05 = throwing
;06 = submerging
;07 = dead (just RTSs after submerging slightly faster)



Wait:
PLX
LDA !Timer,x
BEQ .next           ;decrease timer, if it hits 0 go to next state
RTS

  .next
LDA !Cargo,x
STA !State,x        ;set new state

STZ !Cargo,x


CMP #$01            ;only check direction if next state is emerging
BNE .NoSideSwitch

JSR SubHorzPos
TYA
EOR #$01
STA !Dir,x

  .NoSideSwitch
RTS



Emerge:
PLX

STZ !ArmDex,x   ;reset arms if no sprite is spawned

LDA $14
AND #$07        ;every 8th frame
BNE .NoShuffle

LDA !DirtDir,x
EOR #$01        ;flip the dirt direction
STA !DirtDir,x

  .NoShuffle

LDA !Buried,x
BEQ .DoneRise   ;when we hit the top, next state

DEC !Buried,x   ;otherwise keep going

  .DoneRise
LDA !Cargo,x    ;if we got to the top goal for 09 frames, stop going for 05
CMP #$09
BEQ .ArmSettle

LDA !ArmY,x
CMP #!ArmRaise  ;if arm isnt at goal 1 yet, keep going there
BNE .Keep06

INC !Cargo,x    ;only run this once
BRA .NoArmSettle

  .Keep06
LDA #!ArmRaise
STA !ArmGoal,x
BRA .NoArmSettle

  .ArmSettle
LDA #!ArmRest   ;set goal to on the rocks
STA !ArmGoal,x

LDA !ArmY,x
CMP #!ArmRest
BEQ .next

  .NoArmSettle
RTS


  .next
STZ !State,x    ;wait a bit

LDA #$1F        ;this long
STA !Timer,x

LDA #$02        ;lowering state
STA !Cargo,x
RTS





Lower:
PLX

LDA !ArmY,x
CMP #!Hidden   ;if arms aren't low yet
BEQ .LowerBody

LDA #!Hidden   ;lowering arms
STA !ArmGoal,x
RTS


  .LowerBody
LDA !Buried,x  ;once arms are low, lower body a tiny tad
CMP #$02
BEQ .next

INC !Buried,x
RTS


  .next
STZ !State,x

LDA #$12
STA !Timer,x

LDA #$03      ;wait a tad before entering spawn state (so jumping on it now wont spawn a dumb-ass projectile
STA !Cargo,x

LDA #$01      ;set arms to lifting
STA !ArmDex,x
RTS





Spawn:
;; if you want the sprite to only spawn one of the two sprites, replace the following 
;; 4 lines with LDA #$01 or LDA #$02 for the rock or the bob-omb respectively
JSL $01ACF9     ;RNG generator
LDA $148D       ;load RNG result
AND #$01
INC             ;add 1 to make the 0 or 1 turn into 1 or 2

STA $0D         ;temp store the !Type,x of the new sprite

PLX
JSL $02A9DE     ;get an index to an unused sprite slot, return if all slots full
BMI .NoSlot     ;after: Y has index of sprite being generated

;spawn the projectile here (index in Y)
LDA #$06        ;play gulp sound, good "pulling" sound
STA $1DF9

TYA
INC
STA !ProjDex,x  ;store projectile index + 1
LDA #$00
STA !ProjDex,y  ;tell the other sprite it's being held

LDA #$01        ;init state
STA $14C8,y

LDA $7FAB9E,x
TYX             ;same sprite number
STA $7FAB9E,x

JSL $07F7D2     ;reset sprite properties

LDA $7FAB10,x
ORA #$08        ;mark the sprite as custom
STA $7FAB10,x

LDX $15E9

LDA $0D
STA !Type,y

;positioning for both shatter effect and new sprite
LDA $E4,x
STA $00E4,y
STA $9A
LDA $14E0,x
STA $14E0,y
STA $9B

LDA $D8,x
CLC
ADC #$05
STA $00D8,y
STA $98
LDA $14D4,x
ADC #$00
STA $14D4,y
STA $99

JSR ProjectileBehaviour_ShatterRTN

INC !State,x    ;enter lifting state
RTS

  .NoSlot
LDA #$06        ;make sprite submerge if there's no free slots
STA !State,x
RTS



Lift:
PLX

;keep aiming towards mario
LDA $14
AND #$07
BNE .NoTurn

JSR SubHorzPos  ;store direction
TYA
EOR #$01
STA !Dir,x

  .NoTurn
;;keep rock aligned with hands at all times
LDA $D8,x
CLC
ADC !ArmY,x
STA $0B

LDA !ArmY,x
BMI .AdjustAlign

LDA $14D4,x
ADC #$00
STA $0C
BRA .NoAdjust

  .AdjustAlign
LDA $14D4,x
ADC #$FF
STA $0C

  .NoAdjust
REP #$20
LDA $0B
SEC
SBC #$000B
STA $0B
SEP #$20

LDA !ProjDex,x
DEC
TAY

LDA $14C8,y    ;check if that sprite hasnt exploded yet
CMP #$08
BNE .Sink

LDA $0B
STA $00D8,y
LDA $0C
STA $14D4,y



;shuffle dirt and rise
LDA $14
AND #$07        ;every 8th frame
BNE .NoShuffle

LDA !DirtDir,x
EOR #$01        ;flip the dirt direction
STA !DirtDir,x

  .NoShuffle
LDA !Buried,x
BEQ .DoneRise   ;when we hit the top, next state

DEC !Buried,x   ;otherwise keep going

  .DoneRise
LDA !ArmY,x
CMP #$FD        ;lift hands way above head
BEQ .next

LDA #$FD
STA !ArmGoal,x
RTS

  .Sink
LDA #$06        ;make sprite submerge if the projectile is gone
STA !State,x
RTS



  .next
STZ !State,x

LDA #$10
STA !Timer,x

LDA #$05
STA !Cargo,x

!ThrowTime = $10

LDA.b #!ThrowTime+$10     ;store the throwing timer in there (+18 because we wait that long to switch there)
STA !ThrowTimer,x
RTS





Throw:
PLX

LDA !ThrowTimer,x
BNE +
JMP .next
+
CMP.b #!ThrowTime-$1
BNE .NoInit

;the init, where the sprite is thrown (and checked for)
LDA #$FB
STA !ArmGoal,x

LDA #$FE
STA !Buried,x

LDA #$02
STA !ArmDex,x


LDA !ProjDex,x
DEC
TAY

LDA $14C8,y     ;check if that sprite hasnt exploded yet
CMP #$08
BNE .Sink

LDA #$01        ;tell sprite to have gravity now
STA !ProjDex,y

;give sprite speeds here
PHY
LDA !Dir,x
TAY

LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $94
SEP #$20

BMI .positive
LSR
LSR
STA $04
LSR
LSR
STA $05
LDA $04
SEC
SBC $05

EOR #$FF
INC
BRA .negative

  .positive
EOR #$FF
INC

LSR
LSR
STA $04
LSR
LSR
STA $05
LDA $04
SEC
SBC $05

  .negative
;limit the speeds
CPY #$01
BNE .NegativeSpeed

CMP #$1E
BCC .PositiveSpeed
LDA #$1D

BRA .PositiveSpeed
  .NegativeSpeed

CMP #$E3
BCS .PositiveSpeed
LDA #$E3

  .PositiveSpeed
PLY
STA $00B6,y
LDA #$C0
STA $00AA,y
RTS

  .NoInit
LDA #$10
STA !ArmGoal,x

LDA !ThrowTimer,x
CMP #$08
BCS .Return

INC !ArmY,x
STZ !ArmDex,x
  .Return
RTS

  .Sink
LDA #$06        ;make sprite submerge if the projectile is gone
STA !State,x
RTS


  .next
STZ !State,x

LDA #$18
STA !Timer,x

LDA #$06        ;submerge when done throwing
STA !State,x

RTS




Submerge:
PLX

LDA $14
AND #$07        ;every 8th frame
BNE .NoShuffle

LDA !DirtDir,x
EOR #$01        ;flip the dirt direction
STA !DirtDir,x

  .NoShuffle

STZ !ProjDex,x  ;just in case tell mole that he has nothing in his hands
STZ !ArmDex,x   ;normal hands

LDA !ArmY,x
CMP #!Hidden    ;next state when arms are down
BNE .notnext

LDA !Buried,x
CMP #!Hidden
BEQ .next

  .notnext
LDA #!Hidden
STA !ArmGoal,x

LDA !Buried,x
CMP #!Hidden
BEQ .NoBury     ;when we hit the top, next state

INC !Buried,x   ;otherwise keep going
  .NoBury
RTS


.next
LDA !Dead,x
BNE .deadnext

STZ !State,x

LDA #$48
STA !Timer,x

LDA #$01
STA !Cargo,x
RTS

.deadnext
INC !State,x
RTS




Dead:
PLX
RTS






;death subroutine, not technically a state though it might as well be
Death:
LDA #$06              ;set state to submerging
STA !State,x
LDA #$19              ;make inedible
STA $1686,x

LDA #$01
STA !Dead,x

;;; uncomment this (delete the semi-colons) to make the sprite die forever instead of respawning
LDA #$FF
STA $161A,x

LDA !ProjDex,x        ;if there's a projectile
BEQ .Return

DEC
PHY
TAY
LDA #$02              ;shatter the projectile
STA !ProjDex,y
PLY

  .Return
RTS










ProjectileBehaviour:
LDA !ProjDex,x      ;only have gravity, update speed and run collision with walls when told to
BEQ .NoRelease
CMP #$02            ;or even shatter if told to
BNE .DontDie

JMP .HitWall        ;act as if a wall was hit

  .DontDie
;gravity
LDA $AA,x
INC
INC
BMI .Store
CMP #$38
BCS .DontStore
  .Store
STA $AA,x
  .DontStore

JSL $01801A         ;update x
JSL $018022         ;update y


JSL $019138         ;interact with objects
LDA $1588,x
AND #$04
BNE .HitWall

  .NoRelease        ;where to skip to if mole is holding it
JSL $01A7DC         ;check mario and sprite collision
BCC .NoContact

LDA !Type,x
CMP #$01
BEQ .RockCheck



JSR .Explode
BRA .HurtMario


  .RockCheck
;check if mario is above
LDA $140D             ;check if mario is spinjumping
ORA $187A
BNE +
BRA .HurtMario
+

LDA $7D               ;check if hes falling
BPL +
BRA .HurtMario
+

;move up mario's hitbox by his speed to check for top contact
LDA $01
SEC
SBC $7D
STA $01
LDA $09
SBC #$00
STA $09
JSL $03B72B           ;check if the filled in objects intersect
BCC +
BRA .HurtMario        ;if yes, mario isnt on top, so hurt him
+


;move mario up a bit to not instantly get hit again
LDA $96
SEC
SBC #$03
STA $96
LDA $97
SBC #$00
STA $97


LDA #$02              ;spiny sound
STA $1DF9

JSL $01AB99           ;contact graphic

LDA $15
AND #$80              ;launch mario up depending on if hes holding A/B
BEQ .SmallHop

LDA #$B4
STA $7D
RTS

  .SmallHop
LDA #$D0
STA $7D
RTS


  .HurtMario
LDA $187A
BEQ .DoHurt         ;skip this if mario isnt riding yoshi

JSR LoseYoshi

BRA .NoContact

  .DoHurt
JSL $00F5B7         ;hurt mario

  .NoContact
RTS

  .HitWall
LDA !Type,x
CMP #$01           ;decide whether to shatter or explode
BNE .Explode

;shatter:
  .Shatter
STZ $14C8,x        ;delete sprite


LDA $E4,x
STA $9A
LDA $14E0,x
STA $9B
LDA $D8,x
STA $98             ;transfer sprite pos to block pos
LDA $14D4,x
STA $99


  .ShatterRTN
PHB
LDA #$02            ;prepare bankeronis
PHA
PLB
LDA #$00            ;0 = brown, else rainbow
JSL $028663         ;spawn the effect
PLB
RTS




JSR Smoke
RTS


  .Explode
LDA #$15
STA $1887	        ;shake the ground

LDA #$0D
STA $9E,x	        ;set sprite to bob-omb

LDA #$08	        ;active state
STA $14C8,x

JSL $07F7D2	      ;reset sprite tables


LDA #$01		      ;explosion flag
STA $1534,x

LDA #$40		      ;timer
STA $1540,x

LDA #$1B          ;make invincible to mostly everything mario could kill this with
STA $167A,x

LDA #$09          ;sound effect
STA $1DFC
RTS


;smoke effect at sprite position
Smoke:
LDY #$03
  .SmokeLoop
LDA $17C0,y
BEQ .FoundSmoke     ;check if this slot is empty
DEY
BPL .SmokeLoop
RTS

  .FoundSmoke
LDA #$01            ;puff of smoke effect
STA $17C0,y

LDA #$1C            ;smoke timer
STA $17CC,y

LDA $D8,x           ;y pos
STA $17C4,y
LDA $E4,x           ;x pos
STA $17C8,y
RTS



;score and sound routine
ScoreSounds:
db $13,$14,$15,$16,$17,$18,$19
;all the stompy sounds with increasingly higher pitch

Score:
;JSR here for mario-stomp chain
PHY
LDA $1697               ;load number of enemies stomped, THEN
INC $1697               ;increase consecutive enemies stomped
                        ;this is stupid but it's how SMW sprite expect it to be
BRA Score3_Apply

Score2:
;JSR here for kicked-sprite chain
LDA $1626,y             ;load number of enemies kicked by sprite in y, THEN
PHA
INC
STA $1626,y             ;increase that number
PLA
PHY

BRA Score3_Apply

Score3:
;JSR here for star chain
PHY
LDA $18D2               ;load number of enemies starred, THEN
INC $18D2               ;increase that number

  .Apply
TAY
INY
CPY #$08                ;if consecutive enemies stomped >= 8
BCS .1UPSound           ;don't play sound so 1UP sound can play

LDA ScoreSounds-$1,y    ;play according sound effect
STA $1DF9

  .1UPSound
TYA
CMP #$08                ;if consecutive enemies stomped >= 8, reset to 8
BCC .NoOverflow
LDA #$08

  .NoOverflow
JSL $02ACE5             ;give mario score according to number of enemies
PLY
RTS                     ;return




;;lose yoshi routine (originally by chdata, unstupified by leod)
LoseYoshi:
PHX
LDX $18E2		;Yoshi sprite index +1, would be zero if no Yoshi

LDA #$10
STA $163D,x	;

LDA #$03		;turn off Yoshi drums
STA $1DFA
LDA #$13		;play lose-Yoshi
STA $1DFC

LDA #$02		;set Yoshi to running-away state
STA $C1,x

STZ $187A		;reset riding Yoshi flag
STZ $0DC1		;reset "Mario has Yoshi on OW" flag so he can't keep it anyway
            ;(this is how castle sequences memorize that Yoshi exists)

LDA #$C0		;make Mario jump upwards
STA $7D
STZ $7B			;no x movement

JSR SubHorzPos
LDA .SPEED,y;load yoshi speed indexed by direction
STA $B5,x   ;and make him run

STZ $1593,x		;
STZ $151B,x		;
STZ $18AE		;

LDA #$30		;Mario invincibility timer
STA $1497

PLX			;restore X (sprite index)
RTS

.SPEED
db $E8,$18	;This is Yoshi's X speed table.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GFX Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ProjecTiles:      ;hue clever table name
db !RockTile,!BobTile
;rock, bob-omb

ProjecPals:       ;yxPpcccT properties palette color colour props (for ctrl-f)
db $0D,$0D : db $05,$09
;palettes:
;E, E : A, C

ProjecPP:
db $00,$20,$00
;priorities

FlipDirs:
db $40,$00



  SubGFX:
JSR GET_DRAW_INFO

LDA !Dir,x
TAX
LDA FlipDirs,x  ;store x flip bit to $0E
STA $0E

LDX $15E9

  PHX

LDA !Type,x     ;check if sprite is a mole or a projectile
BEQ MoleGFX


ProjectileGFX:
LDA !ProjDex,x
TAX
LDA ProjecPP,x    ;temp store priorities
STA $0F


LDA $00         ;load sprite x pos
STA $0300,y     ;store to oam x pos

LDA $01         ;load sprite y pos
STA $0301,y     ;store to oam y pos

LDX $15E9
LDA !Type,x     ;load which type of projectile this is
DEC
TAX
LDA ProjecTiles,x    ;load tile by index -1 because type index 0 is monty
STA $0302,y


LDA $14
LSR : LSR
AND #$01        ;flash palette (at least for bob-omb)
STA $0D         ;store 0 or 1 to $0D
TXA
ASL             ;double what we had in x before
ORA $0D         ;and add 0 or 1 to flash
TAX
LDA ProjecPals,x  ;load palettes by index
ORA $0E         ;add in the flip
ORA $0F         ;add the priorities
STA $0303,y


  PLX
  
LDY #$02        ; #$00 = 8x8, #$02 = 16x16, #$FF = variable
LDA #$00        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS





!DirtTile = $88
!DirtPalette = $21
!BodyTile = !MoleTiles
!BodyPalette = $01

XOffs:
db $00,$08


HandTiles:
db !BodyTile+$2,!BodyTile+$3 : db !BodyTile+$12,!BodyTile+$13
db !BodyTile+$4,!BodyTile+$14
;idle, holding projectile
;throw animation left

!HandPalette = $01              



MoleGFX:
;start dirt tiles
Dirt:
LDX #$01
  .Loop
PHX

LDA XOffs,x
CLC
ADC $00             ;x pos = sprite x + offset
STA $0300,y


TXA
LDX $15E9
EOR !DirtDir,x      ;take current dirt tile and switch it with dirt direction, to get different tiles
                    ;if its facing differently
CLC
ADC #!DirtTile      ;add the left dirt tile's coords, so we get either that or that + 1
STA $0302,y


LDA $01
CLC                 ;y pos = sprite y + #$08
ADC #$08
STA $0301,y


LDX $15E9
LDA !DirtDir,x
EOR #$01
TAX
LDA FlipDirs,x
ORA #!DirtPalette   ;yxPpcccT
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;8x8
STA $0460,y    ;store to size table
PLY

INY : INY : INY : INY

PLX
DEX
BPL .Loop
;end dirt tiles

;start hand tiles
Hands:
LDX #$01
  .Loop
PHX

LDA XOffs,x
CLC
ADC $00             ;x pos = sprite x + offset
STA $0300,y


TXA
LDX $15E9
EOR !Dir,x          ;get correct hand tiles according to flip
                    ;if its facing differently
STA $0F

LDA $01
CLC
ADC !ArmY,x         ;y pos = sprite y + arm offset
STA $0301,y


LDA !Dir,x
EOR #$01
TAX
LDA FlipDirs,x
ORA #!HandPalette
STA $0303,y


LDX $15E9
LDA !ArmDex,x
ASL                 ;take armdex * 2
CLC
ADC $0F             ;add direction and loop influenced 1 or 0
TAX                 ;into x
LDA HandTiles,x     ;add the left hand tile's coordinate indexed by armdex,
                    ;so we swap hands according to dir and armdex
STA $0302,y


TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;8x8
STA $0460,y    ;store to size table
PLY

INY : INY : INY : INY

PLX
DEX
BPL .Loop
;end hand tiles

;start body tile
Body:
LDA $00
STA $0300,y           ;x pos = sprite x

LDX $15E9
LDA $01
CLC
ADC !Buried,x         ;y pos = sprite y + buried amount of pixels
STA $0301,y

LDA #!BodyTile
STA $0302,y

LDA !Dir,x
EOR #$01
TAX
LDA FlipDirs,x        ;take flip
ORA #!BodyPalette     ;add palette etc
STA $0303,y


TYA
LSR A
LSR A
PHY
TAY
LDA #$02       ;16x16
STA $0460,y    ;store to size table
PLY
;end body tile

  PLX

LDY #$FF        ; #$00 = 8x8, #$02 = 16x16, #$FF = variable
LDA #$04        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam

RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
        ;org $03B75C
 
TABLE1:              db $0C,$1C
TABLE2:              db $01,$02
 
GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
        STZ $15A0,x             ; reset sprite offscreen flag, horizontal
        LDA $E4,x               ; \
        CMP $1A                 ;  | set horizontal offscreen if necessary
        LDA $14E0,x             ;  |
        SBC $1B                 ;  |
        BEQ ON_SCREEN_X         ;  |
        INC $15A0,x             ; /
 
ON_SCREEN_X:         LDA $14E0,x             ; \
        XBA      ;  |
        LDA $E4,x               ;  |
        REP #$20                ;  |
        SEC      ;  |
        SBC $1A                 ;  | mark sprite INVALID if far enough off screen
        CLC      ;  |
        ADC.w #$0040            ;  |
        CMP.w #$0180            ;  |
        SEP #$20                ;  |
        ROL A                   ;  |
        AND #$01                ;  |
        STA $15C4,x             ; /
        BNE INVALID             ;
       
        LDY #$00                ; \ set up LOOP:
        LDA $1662,x             ;  |
        AND #$20                ;  | if not smushed (1662 & 0x20), go through LOOP twice
        BEQ ON_SCREEN_LOOP      ;  | else, go through LOOP once
        INY      ; /
ON_SCREEN_LOOP:      LDA $D8,x               ; \
        CLC      ;  | set vertical offscreen if necessary
        ADC TABLE1,y            ;  |
        PHP      ;  |
        CMP $1C                 ;  | (vert screen boundry)
        ROL $00                 ;  |
        PLP      ;  |
        LDA $14D4,x             ;  |
        ADC #$00                ;  |
        LSR $00                 ;  |
        SBC $1D                 ;  |
        BEQ ON_SCREEN_Y         ;  |
        LDA $186C,x             ;  | (vert offscreen)
        ORA TABLE2,y            ;  |
        STA $186C,x             ;  |
ON_SCREEN_Y:         DEY         ;  |
        BPL ON_SCREEN_LOOP      ; /
 
        LDY $15EA,x             ; get offset to sprite OAM
        LDA $E4,x               ; \
        SEC      ;  |
        SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
        STA $00                 ; /
        LDA $D8,x               ; \
        SEC      ;  |
        SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
        STA $01                 ; /
        RTS      ; RETURN
 
INVALID:             PLA         ; \ RETURN from *main gfx routine* subroutine...
        PLA      ;  |    ...(not just this subroutine)
        RTS      ; /
		    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS RETURN_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubHorzPos
; Stores direction Sprite is in from Mario in Y
; Stores proximity in $0F
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 